'use strict';

import Gtk from 'gi://Gtk';
import Gio from 'gi://Gio';
import GLib from 'gi://GLib';
import Adw from 'gi://Adw';
import { ExtensionPreferences } from 'resource:///org/gnome/Shell/Extensions/js/extensions/prefs.js';
import { ServerSetting } from './serverSetting.js';
import { ServerGroup } from './serverGroup.js';

export default class ServerStatusPreferences extends ExtensionPreferences {

	fillPreferencesWindow(window) {
		this.page = new Adw.PreferencesPage();		
		this.serverGroups = [];
		this.prefSettings = this.getSettings();

		const helpGroup = new Adw.PreferencesGroup({
			description: `HTTP HEAD is faster than GET but not always supported.\n
If you get a red indicator, try switching to GET.\n
If you get a yellow indicator, there's something wrong with the URL.\n
It should be of format http[s]://host[:port][/path].`,
		});
		this.page.add(helpGroup);

		this.addGroup = new Adw.PreferencesGroup({});
		const addRow = new Adw.ActionRow({
			title: 'Add a new server',
		});
		addRow.connect('activated', () => {
			let newGroup = new ServerGroup(
				window,
				this.page,
				this.saveSettings,
				this.serverGroups,
				this.prefSettings,
				null); // widgets will not be initialized
			this.page.add(newGroup.getGroup());
			this.serverGroups.push(newGroup);
			this.saveSettings(this.prefSettings);
		});
		const addImage = Gtk.Image.new_from_icon_name('list-add-symbolic');
		addRow.add_suffix(addImage);
		addRow.set_activatable_widget(addImage);
		this.addGroup.add(addRow);
		this.page.add(this.addGroup);

		const parsedSettings = this.parseSettings(this.prefSettings);
		for (const savedSettings of parsedSettings) {
			let newGroup = new ServerGroup(
				window,
				this.page,
				this.saveSettings,
				this.serverGroups,
				this.prefSettings,
				savedSettings);
			this.page.add(newGroup.getGroup());
			this.serverGroups.push(newGroup);
		}

		window.add(this.page);
	}

	parseSettings(rawSettings) {
		const variant = rawSettings.get_value('server-settings');
		const savedRawSettings = variant.deep_unpack();
		const settings = [];
		for (const rawSetting of savedRawSettings) {
			const url = rawSetting['url'];
			const frequency = rawSetting['frequency'];
			const is_get = rawSetting['is_get'];
			const setting = new ServerSetting(url, frequency, is_get);
			settings.push(setting);
		}
		return settings;
	}

	saveSettings(prefSettings) {
		const serverSettingList = [];
		for (const serverGroup of this.serverGroups) {
			const settings = serverGroup.getSettings();
			if (settings) {
				settings.url = settings.url.trim();
				settings.frequency = settings.frequency.toString();
				settings.is_get = settings.is_get.toString();
				serverSettingList.push(settings);
			}
		}
		// persist
		prefSettings.set_value(
			'server-settings',
			new GLib.Variant(
				'aa{ss}',
				serverSettingList
			)
		);
		Gio.Settings.sync();
	}
}